/*
 * Copyright 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "BinaryFile.h"

InputBinaryFile::InputBinaryFile(const std::string& filename)
:	in(filename.c_str(), std::ios::binary),
	bufferData(0),
	bufferSize(0)
{	
}

OutputBinaryFile::OutputBinaryFile(const std::string& filename)
:	out(filename.c_str(), std::ios::binary),
	bufferData(0),
	bufferSize(0)
{
}

OutputBinaryFile::~OutputBinaryFile()
{
	if (bufferSize > 0)
	{
		out.write(reinterpret_cast<const char*>(&bufferData), 1);
	}
}

Uint64 InputBinaryFile::read(std::size_t n)
{
	Uint64 value = 0;
	std::size_t shift = 0;
	do
	{
		if (bufferSize == 0)
		{
			if (!in.read(reinterpret_cast<char*>(&bufferData), 1))
			{
				bufferData = 0;
			}
			bufferSize = 8;
		}
		std::size_t rbits = std::min(bufferSize, n);
		value |= (bufferData & ~(~Uint64(0) << rbits)) << shift;
		shift += rbits;
		n -= rbits;
		bufferData >>= rbits;
		bufferSize -= rbits;
	} while (n > 0);
	return value;
}

void OutputBinaryFile::write(std::size_t n, Uint64 value)
{
	do
	{
		std::size_t wbits = std::min(8 - bufferSize, n);
		bufferData |= (value & ~(~Uint64(0) << wbits)) << bufferSize;
		value >>= wbits;
		bufferSize += wbits;
		n -= wbits;
		if (bufferSize == 8)
		{
			out.write(reinterpret_cast<const char*>(&bufferData), 1);
			bufferData = 0;
			bufferSize = 0;
		}
	} while (n > 0);
}

Uint64 InputBinaryFile::readX(std::size_t chunkSize)
{
	Uint64 value = 0;
	std::size_t shift = 0;
	do
	{
		value |= read(chunkSize) << shift;
		shift += chunkSize;
	} while (read(1));
	return value;
}

void OutputBinaryFile::writeX(std::size_t chunkSize, Uint64 value)
{
	do
	{
		write(chunkSize, value);
		value >>= chunkSize;
		write(1, value > 0 ? 1 : 0);
	} while (value > 0);
}

std::string InputBinaryFile::readS()
{
	std::string str;
	char c;
	while ((c = read(7)))
	{
		str += c;
	}
	return str;
}

void OutputBinaryFile::writeS(const std::string& str)
{
	for (std::size_t i = 0; i < str.length(); ++i)
	{
		write(7, str[i]);
	}
	write(7, 0);
}

int calculateBitsNeeded(Uint64 value)
{
	int bits = 1;
	do
	{
		value >>= 1;
		++bits;
	} while (value > 0);
	return bits;
}
